@@ -14,7 +14,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
@@ -81,6 +81,8 @@ $ST->{$MAIN}{$MAIN} = [ 'context', $MAIN ];
 $ST->{$CALL} = {};
 # register Block in symbol table
 $ST->{$BLOCK} = {};
+# register ARGV to Symbol table
+$ST->{$MAIN}{'ARGV'} = value_to_atom([ @ARGV ]);
 
 ###########################################
 # built-in sub map
@@ -120,10 +122,10 @@ our $Spp_map = {
   'and'     => sub { eval_and(@_)     },
   '&&'      => sub { eval_and(@_)     },
   'begin'   => sub { eval_exprs(@_)   },
-  'block'   => sub { ['str', $BLOCK]  },
+  'block'   => sub { $BLOCK           },
   'bool'    => sub { eval_bool(@_)    },
   'break'   => sub { eval_break()     },
-  'call'    => sub { ['str', $CALL]   },
+  'call'    => sub { $CALL            },
   'case'    => sub { eval_case(@_)    },
   'context' => sub { eval_context(@_) },
   'def'     => sub { eval_def(@_)     },
@@ -133,7 +135,6 @@ our $Spp_map = {
   'to-i'    => sub { eval_to_i(@_)    },
   'exit'    => sub { exit()           },
   'exists'  => sub { eval_exists(@_)  },
-  'lambda'  => sub { eval_lambda(@_)  },
   'for'     => sub { eval_for(@_)     },
   'if'      => sub { eval_if(@_)      },
   'is-false'=> sub { eval_is_false(@_)},
@@ -149,33 +150,34 @@ our $Spp_map = {
   'ok'      => sub { eval_ok(@_)      },
   'or'      => sub { eval_or(@_)      },
   '||'      => sub { eval_or(@_)      },
-  'uuid'    => sub { ['str', uuid()]  },
+  'uuid'    => sub { uuid()           },
   'return'  => sub { eval_return(@_)  },
   'rule'    => sub { eval_rule(@_)    },
   'say'     => sub { eval_say(@_)     },
   'set'     => sub { eval_set(@_)     },
   'shift'   => sub { eval_shift(@_)   },
   'trim'    => sub { eval_trim(@_)    },
-  'type'    => sub { eval_type($_[0]) },
-  'where'   => sub { ['str',$CONTEXT] },
+  'type'    => sub { eval_type(@_)    },
+  'value'   => sub { eval_value(@_)   },
   'while'   => sub { eval_while(@_)   },
 };
 
 our $Perl_map = {
   'eval'      => sub { eval_atom(@_)   },
-  'fill'      => sub { fill(@_)        },
+  'fill'      => sub { fill_array(@_)  },
   'first'     => sub { first(@_)       },
   'format'    => sub { sprintf(@_)     },
   'from-json' => sub { decode_json(@_) },
   'index'     => sub { array_index(@_) },
   'join'      => sub { perl_join(@_)   },
   'last'      => sub { tail(@_)        },
-  'length'    => sub { len(@_)         },
+  'len'       => sub { len(@_)         },
   'load_file' => sub { load_file(@_)   },
   'max'       => sub { perl_max(@_)    },
   'opt'       => sub { opt_atom(@_)    },
   'read-file' => sub { read_file(@_)   },
   'rest'      => sub { rest(@_)        },
+  'see'       => sub { see(@_)         },
   'split'     => sub { perl_split(@_)  },
   'substr'    => sub { perl_substr(@_) },
   'to-json'   => sub { encode_json(@_) },
@@ -184,21 +186,12 @@ our $Perl_map = {
   'zip'       => sub { perl_zip(@_)    },
 };
 
-=head1 SUBROUTINES/METHODS
-
-Spp atom format to string to view
-
-=head2 atom_to_str
-
-transfer Spp atom data to string to output.
-
-=cut
-
 sub atom_to_str {
   my $atom = shift;
-  my ($name, $value) = @{$atom};
-  return atoms_to_strs($atom) if is_perl_array($name);
-  given ($name) {
+  my $type = type($atom);
+  my $value = value($atom);
+  return atoms_to_strs($atom) if is_perl_array($type);
+  given ($type) {
     when ('dot')     { return '.' }
     when ('any')     { return '.' }
     when ('nil')     { return 'nil' }
@@ -233,7 +226,7 @@ sub atom_to_str {
     when ('chclass') { return chclass_to_str($value) }
     when ('action')  { return atoms_to_strs($value) }
     when ('char')    { return char_to_str($value) }
-    default { croak("Unknown atom to str: $name") }
+    default { error("Unknown atom to str: $type") }
   }
 }
 
@@ -401,10 +394,10 @@ sub chclass_to_str {
   my $atoms = shift;
   my @chclass_list;
   for my $atom (values $atoms) {
-    if ($atom->[0] eq 'flip') {
+    if (type($atom) eq 'flip') {
       push @chclass_list, '^';
     } else {
-      push @chclass_list, tail($atom);
+      push @chclass_list, value($atom);
     }
   }
   my $chclass_str = join '', @chclass_list;
@@ -423,7 +416,7 @@ sub get_token_atom {
       return value($token) if is_rule($token);
     }
   }
-  croak "token: <$name> not defined!";
+  error "token: <$name> not defined!";
 }
 
 sub match_rule {
@@ -488,7 +481,7 @@ sub out_context {
     $CONTEXT = first($STACK);
     return ['context', $name];
   }
-  croak("Could not end $name from $CONTEXT");
+  error("Could not end $name from $CONTEXT");
 }
 
 sub in_call {
@@ -501,7 +494,7 @@ sub in_call {
     $CONTEXT = $context;
     return ['context', $context];
   }
-  croak("into exists block: $context");
+  error("into exists block: $context");
 }
 
 sub out_call {
@@ -514,7 +507,7 @@ sub out_call {
     $CONTEXT = first($STACK);
     return ['context', $context];
   }
-  croak("Out block $context != $CONTEXT");
+  error("Out block $context != $CONTEXT");
 }
 
 sub in_block {
@@ -527,7 +520,7 @@ sub in_block {
     unshift $BSTACK, $context;
     return ['context', $context];
   }
-  croak("into block exists: $context");
+  error("into block exists: $context");
 }
 
 sub out_block {
@@ -540,14 +533,14 @@ sub out_block {
     $BLOCK = first($BSTACK);
     return ['context', $context];
   }
-  croak("Out block $context != $CONTEXT");
+  error("Out block $context != $CONTEXT");
 }
 
 sub eval_local_declare {
   my ($sym, $value) = @_;
   my $name = value($sym);
   if (exists $ST->{$CONTEXT}{$name}) {
-    croak("symbol Have defined: $name");
+    error("symbol Have defined: $name");
   }
   $ST->{$CONTEXT}{$name} = $value;
   return $sym;
@@ -557,13 +550,13 @@ sub eval_multi_local_declare {
   my ($syms, $values) = @_;
   return ['true'] if len($syms) == 0;
   if (all_is_sym($syms)) {
-    for my $sym_value (values zip($syms, $values)) {
+    for my $sym_value (values perl_zip($syms, $values)) {
       my ($sym, $value) = @{$sym_value};
       eval_local_declare($sym, $value);
     }
     return ['list', $syms];
   }
-  croak("only could bind symbol: $syms");
+  error("only could bind symbol: $syms");
 }
 
 sub eval_sym_assign {
@@ -575,7 +568,7 @@ sub eval_sym_assign {
       return $sym;
     }
   }
-  croak("Assign undefined symbol: $name");
+  error("Assign undefined symbol: $name");
 }
 
 sub eval_syms_assign {
@@ -592,14 +585,15 @@ sub to_rule {
   my $parse_str = read_file($grammar_file);
   my $match_ast = match_rule($RULE, $parse_str);
   if (is_match($match_ast)) {
+    # see $match_ast;
     my $opt_ast  = opt_atom($match_ast);
-    my $rule_ast = value_to_atom($opt_ast);
-    my $eval_ast = eval_atom($rule_ast);
-    my $rule_str = atom_to_str($eval_ast);
-    write_file($rule_file, $rule_str);
+    if (is_same($opt_ast, $Spp_rule)) {
+      say "ok, to-rule make same rule with spp rule";
+    }
+    write_file($rule_file, to_str($opt_ast));
     return ['true'];
   }
-  croak("Could not transfer $grammar_file to rules");
+  error("Could not transfer $grammar_file to rules");
 }
 
 ##########################################
@@ -620,9 +614,9 @@ sub name_match {
   my ( $name, $match ) = @_;
   return ['false']         if is_fail($match);
   return ['true']          if is_true($match);
-  return [$name, $match]   if is_perl_str($match);
+  return [$name, $match]   if is_str($match);
   return $match            if $name =~ /^[a-z_]/;
-  return [$name, [$match]] if is_atom($match);
+  return [$name, [$match]] if is_match_atom($match);
   return [$name, $match];
 }
 
@@ -630,22 +624,22 @@ sub gather_match {
   my ( $gather, $match ) = @_;
   return $match if is_fail($match);
   return $match if is_true($gather);
-  if ( is_perl_str($gather) ) {
+  if ( is_str($gather) ) {
     if ( is_true($match) ) { return $gather }
-    if ( is_perl_str($match) ) { return $gather . $match }
-    if ( is_atom($match) )  { return $match }
-    if ( is_atoms($match) ) { return $match }
+    if ( is_str($match) ) { return $gather . $match }
+    if ( is_match_atom($match) )  { return $match }
+    if ( is_match_atoms($match) ) { return $match }
   }
-  if (is_true($match) or is_perl_str($match)) { return $gather }
-  if ( is_atom($gather) ) {
-    if ( is_atom($match) ) { return [ $gather, $match ] }
-    if ( is_atoms($match) ) { return [ $gather, @{$match} ] }
+  if (is_true($match) or is_str($match)) { return $gather }
+  if ( is_match_atom($gather) ) {
+    if ( is_match_atom($match) ) { return [ $gather, $match ] }
+    if ( is_match_atoms($match) ) { return [ $gather, @{$match} ] }
   }
-  if ( is_atoms($gather) ) {
-    if ( is_atom($match) ) { return [ @{$gather}, $match ] }
-    if ( is_atoms($match) ) { return [ @{$gather}, @{$match} ] }
+  if ( is_match_atoms($gather) ) {
+    if ( is_match_atom($match) ) { return [ @{$gather}, $match ] }
+    if ( is_match_atoms($match) ) { return [ @{$gather}, @{$match} ] }
   }
-  croak([$gather,$match]);
+  error([$gather,$match]);
 }
 
 ######################
@@ -654,12 +648,13 @@ sub gather_match {
 
 sub _match_atom {
   my ($atom, $cursor) = @_;
-  my ($name, $value) = @{$atom};
-  given ($name) {
+  my $type = type($atom);
+  my $value = value($atom);
+  given ($type) {
     when ('true')    { return ['true']  }
     when ('false')   { return ['false'] }
     when ('any')     { return match_any($cursor) }
-    when ('char')    { return match_char($value, $cursor)   }
+    when ('char')    { return match_str($value, $cursor)   }
     when ('str')     { return match_str($value, $cursor)    }
     when ('token')   { return match_token($value, $cursor)  }
     when ('group')   { return match_token($value, $cursor)  }
@@ -676,7 +671,7 @@ sub _match_atom {
     when ('gtoken')  { return match_gtoken($value, $cursor) }
     when ('assert')  { return match_assert($value, $cursor) }
     when ('cclass')  { return match_cclass($value, $cursor) }
-    default { croak("Unknown match atom name: $name") }
+    default { error("Unknown match atom name: $type") }
   }
 }
 
@@ -856,14 +851,15 @@ sub match_rept {
 sub match_action {
   my ($exprs, $cursor) = @_;
   my $exprs_value = eval_atom($exprs);
-  my ($type, $value) = @{ $exprs_value };
+  my $type = type($exprs_value);
+  my $value = value($exprs_value);
   given ($type) {
     when ('nil')   { return ['false']                    }
     when ('true')  { return ['true']                     }
     when ('false') { return ['false']                    }
     when ('str')   { return match_str($value, $cursor)   }
     when ('array') { return match_array($value, $cursor) }
-    default { croak("Not implement action: $type")       }
+    default { error("Not implement action: $type")       }
   }
 }
 
@@ -897,16 +893,6 @@ sub match_str {
   return ['false']
 }
 
-sub match_char {
-  my ($char, $cursor) = @_;
-  my $apply_char = apply_char(1, $cursor);
-  if ($char eq $apply_char) {
-    $cursor->{POS}++;
-    return $char;
-  }
-  return ['false'];
-}
-
 sub match_chclass {
   my ($atom, $cursor) = @_;
   my $char = apply_char(1, $cursor);
@@ -936,16 +922,18 @@ sub match_assert {
       return bool($cursor->{POS} >= $cursor->{LEN});
     }
     when ('^^') {
-      return ['true'] if bool(apply_char(-1, $cursor) =~ /\n/);
-      return ['true'] if bool($cursor->{POS} == 1);
+      # say $cursor->{STR};
+      # say ord(apply_char(-1, $cursor));
+      return ['true'] if apply_char(-1, $cursor) =~ /\n/;
+      return ['true'] if $cursor->{POS} == 0;
       return ['false'];
     }
     when ('$$') {
-      return ['true'] if bool(apply_char(1, $cursor) =~ /\n/);
-      return ['true'] if bool($cursor->{POS} >= $cursor->{LEN});
+      return ['true'] if apply_char(1, $cursor) =~ /\n/;
+      return ['true'] if $cursor->{POS} >= $cursor->{LEN};
       return ['false'];
     }
-    default { croak("Unknown assert str: $str") }
+    default { error("Unknown assert str: $str") }
   }
 }
 
@@ -955,8 +943,9 @@ sub match_assert {
 
 sub eval_atom {
   my $atom = shift;
-  my ($name, $value) = @{ $atom };
-  given ($name) {
+  my $type = type($atom);
+  my $value = value($atom);
+  given ($type) {
     when ('list')   { return eval_list($value)   }
     when ('sym')    { return eval_sym($value)    }
     when ('string') { return eval_string($value) }
@@ -982,7 +971,7 @@ sub eval_list {
     my $op_name = value(second($atoms));
     if (exists $Op_map->{$op_name}) {
       my $op_call = $Op_map->{$op_name};
-      if (is_perl_func($op_call)) {
+      if (is_func($op_call)) {
         my $args = [ first($atoms), tail($atoms) ];
         return $op_call->($args);
       }
@@ -992,7 +981,7 @@ sub eval_list {
   if (len($atoms) > 2 and is_if($atoms->[-2])) {
     my $cond_expr = tail($atoms);
     return ['false'] if is_fail(eval_atom($cond_expr));
-    my $true_expr = sublist($atoms, 0, -3);
+    my $true_expr = subarray($atoms, 0, -3);
     return eval_list($true_expr);
   }
 
@@ -1001,7 +990,7 @@ sub eval_list {
     my $name = value($head_atom);
     if (exists $Spp_map->{$name}) {
       my $eval_call = $Spp_map->{$name};
-      if (is_perl_func($eval_call)) {
+      if (is_func($eval_call)) {
         my $args = rest($atoms);
         return $eval_call->($args);
       }
@@ -1009,8 +998,9 @@ sub eval_list {
 
     if (exists $Perl_map->{$name}) {
       my $perl_call = $Perl_map->{$name};
-      if (is_perl_func($perl_call)) {
+      if (is_func($perl_call)) {
         my $args = eval_args(rest($atoms));
+        # see $args;
         my $values = atoms_to_value($args);
         my $return_value = $perl_call->(@{$values});
         return value_to_atom($return_value);
@@ -1041,7 +1031,7 @@ sub eval_list {
   if (is_context($head_value)) {
     if ( is_dot(second($atoms)) ) {
       my $context = value($head_value);
-      my $args = sublist($atoms, 2, -1);
+      my $args = subarray($atoms, 2, -1);
       in_context($context);
       my $return_value = eval_list($args);
       out_context($context);
@@ -1063,8 +1053,8 @@ sub eval_list {
     # hash call use ['hash', ...]
     return hash_call($head_value, $args);
   }
-  my $head_value_type = type($head_value);
-  croak("Have not implement $head_value_type");
+  my $head_value_str = value($head_value);
+  error("Have not implement ($head_value_str ..)");
 }
 
 # if exists sym records, return its value
@@ -1090,7 +1080,7 @@ sub eval_string {
       push $strs, atom_to_str($value);
     }
   }
-  return [ 'str', perl_join($strs) ];
+  return perl_join($strs);
 }
 
 sub eval_array {
@@ -1158,7 +1148,7 @@ sub str_call{
   if (is_int(first($nums))) {
     my $index = value(first($nums));
     if (len($nums) == 1) {
-      return ['str', substr($str, $index, 1)];
+      return substr($str, $index, 1);
     }
     if (len($nums) == 2 and is_int(tail($nums))) {
       my $to_index = value(tail($nums));
@@ -1166,10 +1156,10 @@ sub str_call{
       if ($to_index < 0) {
         $str_len = len($str) + $to_index + 1 - $index;
       }
-      return ['str', substr($str, $index, $str_len)];
+      return substr($str, $index, $str_len);
     }
   }
-  croak("syntax croak str call");
+  error("syntax error str call");
 }
 
 # (array 1) (array 1 2) (array 2 -3) (array [1 2]
@@ -1191,7 +1181,7 @@ sub array_call {
       if ($to_index < 0) {
         $len_array = len($array_value) + $to_index + 1 + $index;
       }
-      my $elements = sublist($array_value, $index, $len_array);
+      my $elements = subarray($array_value, $index, $len_array);
       return ['nil'] if $elements eq '';
       return ['array', $elements];
     }
@@ -1291,7 +1281,7 @@ sub eval_set {
       error("Spp only implement set <Hash> value");
     }
   }
-  croak("(set syntax error: argument less than 3)");
+  error("(set syntax error: argument less than 3)");
 }
 
 # add rule value to symbol table
@@ -1302,14 +1292,20 @@ sub eval_rule {
   if (is_sym($name) and is_rule($rule)) {
     return eval_local_declare($name, $rule);
   }
-  croak("Syntax error: (rule .. )");
+  error("Syntax error: (rule .. )");
 }
 
 # get Spp value type
 sub eval_type {
   my $args = shift;
   my $atom = first($args);
-  return ['str', type(eval_atom($atom))];
+  return type(eval_atom($atom));
+}
+
+sub eval_value {
+  my $args = shift;
+  my $atom = first($args);
+  return value_to_atom(value(eval_atom($atom)));
 }
 
 sub eval_is {
@@ -1323,7 +1319,7 @@ sub eval_is {
     }
     return ['false'];
   }
-  croak("could not compare $type_str, only accept str");
+  error("could not compare $type_str, only accept str");
 }
 
 sub eval_shift {
@@ -1337,7 +1333,7 @@ sub eval_shift {
     # if could not dethory data, should make an copy of array
     return ['array', $array_value];
   }
-  croak("could not shift $array_type");
+  error("could not shift $array_type");
 }
 
 sub eval_say {
@@ -1353,11 +1349,13 @@ sub eval_say {
     say atom_to_str(eval_atom($atom));
     return ['true'];
   }
-  croak("(say with blank argrument");
+  error("(say with blank argrument");
 }
 
 sub eval_context {
   my $args = shift;
+  # (context) return current CONTEXT
+  return $CONTEXT if len($args) == 0;
   my $atom = first($args);
   my $context = eval_atom($atom);
   my $type_str = type($context);
@@ -1366,7 +1364,7 @@ sub eval_context {
     my $context_name = tail($context);
     return in_context($context_name);
   }
-  croak("(context .. ) syntax croak");
+  error("(context .. ) syntax error");
 }
 
 sub eval_assign {
@@ -1386,7 +1384,7 @@ sub eval_assign {
       return eval_syms_assign($syms, $values);
     }
   }
-  croak("assign syntax croak: $atoms");
+  error("assign syntax error: $atoms");
 }
 
 sub eval_end {
@@ -1396,9 +1394,10 @@ sub eval_end {
     if ($CONTEXT eq tail($atom)) {
       return out_context($CONTEXT);
     }
+    error("out context is not $CONTEXT")
   }
   my $atom_type = type($atom);
-  croak("Syntax of (end ..): not context $atom_type");
+  error("Syntax of (end ..): not context $atom_type");
 }
 
 sub eval_defined {
@@ -1412,16 +1411,16 @@ sub eval_defined {
     }
     return ['false'];
   }
-  croak("Symtax croak: (defined ..)");
+  error("Symtax error: (defined ..)");
 }
 
 sub eval_trim {
   my $args = shift;
   my $str_atom = first($args);
   if (is_str($str_atom)) {
-    return ['str', trim(value($str_atom))];
+    return trim(value($str_atom));
   }
-  croak "Trim only support String";
+  error "Trim only support String";
 }
 
 sub eval_exists {
@@ -1549,7 +1548,7 @@ sub eval_delete {
     }
     return ['false'];
   }
-  croak("Spp only implement delete symbol or hash key");
+  error("Spp only implement delete symbol or hash key");
 }
 
 sub eval_case {
@@ -1563,12 +1562,12 @@ sub eval_case {
       if (is_when(first($branch_exprs))) {
         my $cond_atom = $branch_exprs->[1];
         if (is_same($case_value, $cond_atom)) {
-          my $true_exprs = sublist($branch_exprs, 2, -1);
+          my $true_exprs = subarray($branch_exprs, 2, -1);
           return eval_exprs($true_exprs);
         }
       }
       if ( is_else($branch_exprs) ) {
-        my $true_exprs = sublist($branch_exprs, 1, -1);
+        my $true_exprs = subarray($branch_exprs, 1, -1);
         return eval_exprs($true_exprs);
       }
     }
@@ -1578,7 +1577,7 @@ sub eval_case {
 
 sub eval_if {
   my $atoms = shift;
-  if (len($atoms) < 2 ) { croak("if syntax croak: elements < 2") }
+  if (len($atoms) < 2 ) { error("if syntax error: elements < 2") }
   my $cond_expr = $atoms->[0];
   my $true_atoms = [ $atoms->[1] ];
   my $index = 3;
@@ -1588,14 +1587,14 @@ sub eval_if {
       if (is_true($cond_expr)) {
         return eval_exprs($true_atoms);
       }
-      my $if_exprs = sublist($atoms, $index+1, -1);
+      my $if_exprs = subarray($atoms, $index+1, -1);
       return eval_if($if_exprs);
     }
     if (is_else($atom)) {
       if (is_true($cond_expr)) {
         return eval_atom($true_atoms);
       }
-      my $else_atoms = sublist($atoms, $index+1, -1);
+      my $else_atoms = subarray($atoms, $index+1, -1);
       return eval_exprs($else_atoms);
     } else {
       push($true_atoms, $atom);
@@ -1608,7 +1607,7 @@ sub eval_if {
 
 sub eval_while {
   my $atoms = shift;
-  if (len($atoms) < 2) { croak('(while ...) args less 2') }
+  if (len($atoms) < 2) { error('(while ...) args less 2') }
   my $guide_expr = first($atoms);
   my $while_exprs = rest($atoms);
   my $return_value = ['true'];
@@ -1633,7 +1632,7 @@ sub eval_while {
 sub eval_for {
   my $atoms = shift;
   if (len($atoms) < 4) {
-    croak("syntax croak for: args less 4");
+    error("syntax error for: args less 4");
   }
   my ($var, $in, $atom, @exprs) = @{ $atoms };
   my $exprs = [ @exprs ];
@@ -1652,7 +1651,7 @@ sub eval_for {
       }
     }
   }
-  croak('for syntax croak ..');
+  error('for syntax error ..');
 }
 
 sub eval_for_array {
@@ -1705,34 +1704,35 @@ sub eval_for_hash {
 
 sub eval_def {
   my $atoms = shift;
-  my $head_atom = first($atoms);
-  if (is_sym($head_atom)) {
+  my $def_name = first($atoms);
+  if (is_sym($def_name)) {
     my $lambda_exprs = eval_lambda(rest($atoms));
-    return eval_local_declare($head_atom, $lambda_exprs);
+    return eval_local_declare($def_name, $lambda_exprs);
   }
-  if (is_list($head_atom)) {
+  if (is_list($def_name)) {
     my $lambda_exprs = eval_lambda($atoms);
     return $lambda_exprs;
   }
-  croak("define syntax (#{atoms})");
+  error("define syntax (#{atoms})");
 }
 
 sub eval_lambda {
   my $atoms = shift;
-  if (len($atoms) < 2) { croak('define accept 2 argment') }
+  # see $atoms;
+  if (len($atoms) < 2) { error('define accept 2 argment') }
   my $lambda_exprs = rest($atoms);
-  if (len($atoms) == 0) { croak('define accept 1 exprssion') }
-  if (is_list($atoms->[0])) {
-    my $args = $atoms->[0][1];
+  if (len($atoms) == 0) { error('define accept 1 exprssion') }
+  if (is_list(first($atoms))) {
+    my $args = value(first($atoms));
     if (len($args) == 0) {
       return ['lambda', [[], $lambda_exprs]];
     }
     if (all_is_sym($args)) {
       return ['lambda', [$args, $lambda_exprs]];
     }
-    croak("lambda args should is symbols");
+    error("lambda args should is symbols");
   }
-  croak("lambda args should is list");
+  error("lambda args should is list");
 }
 
 sub eval_ok {
@@ -1807,10 +1807,10 @@ sub eval_my {
         my $values = tail($array);
         return eval_multi_local_declare($syms, $values);
       }
-      croak("my value not array: $array");
+      error("my value not array: $array");
     }
   }
-  croak("my syntax croak: { atoms_to_strs(atoms) }");
+  error("my syntax error: { atoms_to_strs(atoms) }");
 }
 
 sub eval_inc {
@@ -1835,7 +1835,7 @@ sub eval_load {
       return eval_atom(opt_atom($match_ast));
     }
   }
-  croak("Load only could accept file name");
+  error("Load only could accept file name");
 }
 
 sub eval_import {
@@ -1851,13 +1851,13 @@ sub eval_import {
           my $ref = ['ref', [$c_name, $sym]];
           $ST->{$CONTEXT}{$name} = $ref;
         } else {
-          croak("Not find symbol: $name in: $c_name");
+          error("Not find symbol: $name in: $c_name");
         }
       }
       return ['list', value($syms)];
     }
   }
-  croak("import only support context name");
+  error("import only support context name");
 }
 
 sub eval_add {
@@ -1865,10 +1865,10 @@ sub eval_add {
   my $atoms = eval_args($args);
   my $values = [ map { value($_) } @{$atoms} ];
   if (all_is_str($atoms)) {
-    return [ 'str', perl_join($values) ];
+    return perl_join($values);
   }
   if (all_is_int($atoms)) {
-    return ['int', perl_sum($values) ];
+    return perl_sum($values);
   }
   if (all_is_array($atoms)) {
     return ['array', perl_concat($values) ];
@@ -1972,16 +1972,14 @@ sub eval_match {
   my $type_str = type($str);
   my $type_rule = type($rule);
   if ($type_str eq 'str') {
-    my $match_str = value($str);
     if ($type_rule eq 'str') {
-      my $regex = value($rule);
-      if ($match_str =~ /$regex/) {
+      if ($str =~ /$rule/) {
         return ['true'];
       }
       return ['false'];
     }
     if ($type_rule eq 'rule') {
-      my $cursor = create_cursor($match_str);
+      my $cursor = create_cursor($str);
       my $rule_name = value($rule);
       my $match = match_atom($rule_name, $cursor);
       if (is_false($match)) { return ['false'] };
@@ -1989,7 +1987,7 @@ sub eval_match {
     }
     if ($type_rule eq 'context') {
       my $context_name = value($rule);
-      my $match = match_rule($context_name, $match_str);
+      my $match = match_rule($context_name, $str);
       if (is_false($match)) { return ['false'] }
       return value_to_atom($match);
     }
@@ -2025,10 +2023,15 @@ sub spp {
   my $spp_str = trim(read_file($spp_file));
   boot_spp();
   my $match = match_rule($RULE, $spp_str);
+  return ['nil'] if is_str($match);
   if (is_match($match)) {
+    # see $match;
     my $opt_ast = opt_atom($match);
-    eval_atom($opt_ast);
+    # see $opt_ast;
+    return atom_to_str(eval_atom($opt_ast));
   }
+  # error("Could not parse $spp_str");
+  return ['false'];
 }
 
 sub repl {
@@ -2043,7 +2046,9 @@ sub repl {
     my $match = match_rule($RULE, $str);
     $DEBUG = 0;
     if (is_match($match)) {
+      # see $match;
       my $opt_ast = opt_atom($match);
+      # see $opt_ast;
       my $eval_value = eval_atom($opt_ast);
       say atom_to_str($eval_value);
     } else {
